==> libremail-2.2.4.tar.bz2: libremail/src/smtp.c <==
/*
    Fichier smtp.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Bibliothque de fonctions permettant
    d'envoyer des mails au serveur smtp
*/

#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include <netinet/in.h>
#include "buflect.h"
#include "smtp.h"
#include "base64.h"


// #define DEBUG

#define defaut_port_smtp  25  // port utilis par dfaut par les serveurs smtp

#define octet      unsigned char


/* variable globale au source
   (pour viter des tonnes de passages de paramtres) */

static int smtp = -1;         // descripteur pour dialoguer avec le serveur



/* connexion au serveur smtp indiqu dans le fichier de configuration */

int connect_smtp (char *infoserveur)
{
    octet *carinfo;       // caractre de infoserveur
    char  serv_smtp [80]; // nom du serveur smtp utilis
    int   port;           // numro du port smtp utilis
    FILE  *fich_auth;     // descripteur du fichier d'authentification smtp
    char  bufin [80];     // buffer pour lire le fichier d'authentification
    char  bufw [80];      // buffer d'envoi d'une ligne ou requte smtp
    int   caract;         // caractre du fichier d'authentification smtp
    int   retour;         // code de retour de la fonction


    // initialisation du numro de port par dfaut
    port = defaut_port_smtp;

    // si la ligne smtp de fichier de configuration fait
    // rfrence  un fichier d'authentification smtp
    if (*infoserveur == '>')
    {
        // se positionner sur le nom de ce fichier
        carinfo = infoserveur + 1;

        while (*carinfo == ' ' || *carinfo == '\t')
            carinfo ++;

        fich_auth = fopen (carinfo, "r");

        if (fich_auth)
        {
            // recuprer le nom du serveur smtp
            fgets (serv_smtp, 80, fich_auth);
            serv_smtp [strlen (serv_smtp) - 1] = '\0';

            // mmoriser les autres lignes particulires du fichier
            // d'authentification smtp (on s'arrte sur une ligne vide)
            while (fgets (bufin, 80, fich_auth) && *bufin != '\n')
            {
                bufin [strlen (bufin) - 1] = '\0';

                // numro de port spcifi ?
                if (startligne (bufin, "port"))
                {
                    // rcuprer le numro du port
                    carinfo = bufin + 4;

                    while (*carinfo == ' ' || *carinfo == '\t')
                        carinfo++;

                    port = atoi (carinfo);

                    // le vrifier sommairement
                    if (!port)
                    {
                        // "Numro de port smtp invalide"
                        affiche_err ("PORT_SMTP_INVALIDE");
                        retour = 0;
                    }
                }
            }

            // tentative de connexion au serveur smtp
            if (port)
                retour = connect_serv_smtp (serv_smtp, port);

            // si la connexion s'est bien droule
            if (retour)
            {
                // se positionner sur la premire ligne
                // du dialogue d'authentification
                while (fgets (bufin, 80, fich_auth) && *bufin == '\n')
                    ;

                // dialogue d'authentification
                do
                {
                    // mise en forme de la ligne lue
                    bufin [strlen (bufin) - 1] = '\0';

                    // si ligne  encoder base 64
                    if (startligne (bufin, "b64"))
                    {
                        // l'encoder avant de l'envoyer
                        encode64 (bufin + 4, bufw, strlen (bufin + 4));
#ifdef DEBUG
                        puts (bufin);
#endif
                        envoie_smtp (bufw);
                    }
                    // sinon, envoyer directement la ligne
                    else
                        envoie_smtp (bufin);

                    // et lire la rponse du serveur
                    lire_smtp ();
                }
                // passage  la ligne suivante
                while (fgets (bufin, 80, fich_auth));
            }

            // librer le fichier d'authentification smtp
            fclose (fich_auth);
        }
        else
        {
            // message d'erreur
            // "Fichier %s non trouv"
            aff_err_arg ("FICH_ABSENT", carinfo);
            retour = 0;
        }
    }
    // sinon simple connexion  un serveur smtp sans authentification
    else
    {
        // parcourir le nom du serveur smtp
        carinfo = infoserveur;

        while (*carinfo > ' ')
            carinfo ++;

        // si le nom du serveur smtp est suivi d'autres informations
        if (*carinfo)
        {
            // terminer le nom du serveur smtp
            *carinfo = '\0';

            // se positionner sur le numro de port ventuel
            do
                carinfo++;
            while (*carinfo == ' ' || *carinfo == '\t');
        }

        // si un numro de port semble prsent
        if (*carinfo)
            // le rcuprer
            port = atoi (carinfo);

        // si numro de port trouv ou si on a conserv le port par dfaut
        if (port)
            // tablir la connexion smtp avec le port choisi
            retour = connect_serv_smtp (infoserveur, port);

        // sinon message d'erreur
        else
        {
            // "Numro de port smtp invalide"
            affiche_err ("PORT_SMTP_INVALIDE");
            retour = 0;
        }

        // si la connexion smtp s'est bien droule
        if (retour)
        {
            // gnrer le message  helo nom_de_calculateur
            strcpy (bufw, "HELO ");
            gethostname (bufw + 5, sizeof (bufw) - 5);

            // identification de l'ordinateur connect
            envoie_smtp (bufw);
            lire_smtp ();
        }
    }

    // renvoyer le code de retour de la fonction
    return (retour);
}



/* connexion au serveur smtp */

int connect_serv_smtp (char *serveur, int port_smtp)
{
    struct hostent *HostEnt;      // description du host serveur
    struct sockaddr_in serv_addr; // addresse du serveur


#ifdef DEBUG
    printf ("Appel de connect_serv_smtp (%s, %d)\n", serveur, port_smtp);
#endif

    // rcupration adresse du serveur  partir de son nom
    HostEnt = gethostbyname (serveur);

    if (HostEnt == NULL)
    {
        // "Serveur smtp non trouv"
        affiche_err ("SERV_SMTP_ABSENT");
        return (0);
    }

    // initialisation structure serv_adr
    memset (&serv_addr, 0, sizeof (serv_addr)); // init serv_addr

    memcpy (&serv_addr.sin_addr, HostEnt->h_addr, HostEnt->h_length);

    serv_addr.sin_port = htons (port_smtp);     // port smtp utilis
    serv_addr.sin_family = AF_INET;             // AF_*** : INET=internet

    // cration de la socket
    if ((smtp = socket (AF_INET, SOCK_STREAM, 0)) < 0)
    {
        // "Problme cration socket client smtp"
        affiche_err ("CREAT_SOCKET_SMTP");
        return (0);
    }

    // requte de connexion
    if (connect (smtp, (struct sockaddr*) &serv_addr, sizeof (serv_addr)) < 0)
    {
        // "Problme demande de connexion"
        // "peut tre due  une connexion Internet par proxy"
        affiche_err ("PB_ACCES_SERV-1");
        affiche_err ("PB_ACCES_SERV-2");
        return (0);
    }
    else
    {
        // attendre la rponse du serveur smtp
        lire_smtp (smtp);
        return (1);
    }
}



/* cherche si la ligne passe en paramtre commence par un mot cl particulier */

int startligne (char *ligne, char *motcle)
{
    int i;

    i = 0;

    // on teste caractre par caractre en ignorant la casse
    while (motcle [i] && tolower (ligne [i]) == tolower (motcle [i]))
        i++;

    // le mot cl doit tre suivi d'un blanc dans la ligne lue
    return (ligne [i] == ' ');
}



/* Ecriture de donnes
   envoie la chaine passe en paramtre suivie de \r\n
*/

void envoie_smtp (char *buffer)
{
#ifdef DEBUG
    putchar ('>');
    puts (buffer);
    fflush (stdout);
#endif

    write (smtp, buffer, strlen (buffer));
    write (smtp, "\r\n", 2);
}



/* Lecture d'une ligne de donnes
   la lecture s'arrte sur un caractre de passage  la ligne
*/

void lire_smtp ()
{
    int posbuf;


    // initialisation
    posbuf = 0;

    // lecture
    do
        read (smtp, buf_lect + posbuf, 1);
    while (buf_lect [posbuf++] != '\n' && posbuf < sz_buflect);

#ifdef DEBUG
    buf_lect [posbuf - 1] = '\0';
    putchar ('<');
    puts (buf_lect);

    // si encodage base64, on affiche aussi la donne en clair
    if (startligne (buf_lect, "334"))
    {
        decode64 (buf_lect + 4);
        puts (buf_lect);
    }

    fflush (stdout);
#endif
}



/* dconnexion propre du serveur smtp */

void deconnect_smtp ()
{
    // envoi du message de deconnexion
    envoie_smtp ("QUIT");

    // lecture de l'acquittement
    lire_smtp ();

    // fermeture de la connexion
    shutdown (smtp, 2);
    close (smtp);
}

==> libremail-2.3.tar.bz2: libremail/src/smtp.c <==
/*
    Fichier smtp.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Bibliothque de fonctions permettant
    d'envoyer des mails au serveur smtp
*/

#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include <netinet/in.h>
#include "buflect.h"
#include "smtp.h"
#include "base64.h"


// #define DEBUG

#define defaut_port_smtp  25  // port utilis par dfaut par les serveurs smtp

#define octet      unsigned char


/* variable globale au source
   (pour viter des tonnes de passages de paramtres) */

static int smtp = -1;         // descripteur pour dialoguer avec le serveur



/* connexion au serveur smtp indiqu dans le fichier de configuration */

int connect_smtp (char *infoserveur)
{
    octet *carinfo;       // caractre de infoserveur
    char  serv_smtp [80]; // nom du serveur smtp utilis
    int   port;           // numro du port smtp utilis
    FILE  *fich_auth;     // descripteur du fichier d'authentification smtp
    char  bufin [80];     // buffer pour lire le fichier d'authentification
    char  bufw [80];      // buffer d'envoi d'une ligne ou requte smtp
    int   caract;         // caractre du fichier d'authentification smtp
    int   retour;         // code de retour de la fonction


    // initialisation du numro de port par dfaut
    port = defaut_port_smtp;

    // si la ligne smtp de fichier de configuration fait
    // rfrence  un fichier d'authentification smtp
    if (*infoserveur == '>')
    {
        // se positionner sur le nom de ce fichier
        carinfo = infoserveur + 1;

        while (*carinfo == ' ' || *carinfo == '\t')
            carinfo ++;

        fich_auth = fopen (carinfo, "r");

        if (fich_auth)
        {
            // recuprer le nom du serveur smtp
            fgets (serv_smtp, 80, fich_auth);
            serv_smtp [strlen (serv_smtp) - 1] = '\0';

            // mmoriser les autres lignes particulires du fichier
            // d'authentification smtp (on s'arrte sur une ligne vide)
            while (fgets (bufin, 80, fich_auth) && *bufin != '\n')
            {
                bufin [strlen (bufin) - 1] = '\0';

                // numro de port spcifi ?
                if (startligne (bufin, "port"))
                {
                    // rcuprer le numro du port
                    carinfo = bufin + 4;

                    while (*carinfo == ' ' || *carinfo == '\t')
                        carinfo++;

                    port = atoi (carinfo);

                    // le vrifier sommairement
                    if (!port)
                    {
                        // "Numro de port smtp invalide"
                        affiche_err ("PORT_SMTP_INVALIDE");
                        retour = 0;
                    }
                }
            }

            // tentative de connexion au serveur smtp
            if (port)
                retour = connect_serv_smtp (serv_smtp, port);

            // si la connexion s'est bien droule
            if (retour)
            {
                // se positionner sur la premire ligne
                // du dialogue d'authentification
                while (fgets (bufin, 80, fich_auth) && *bufin == '\n')
                    ;

                // dialogue d'authentification
                do
                {
                    // mise en forme de la ligne lue
                    bufin [strlen (bufin) - 1] = '\0';

                    // si ligne  encoder base 64
                    if (startligne (bufin, "b64"))
                    {
                        // l'encoder avant de l'envoyer
                        encode64 (bufin + 4, bufw, strlen (bufin + 4));
#ifdef DEBUG
                        puts (bufin);
#endif
                        envoie_smtp (bufw);
                    }
                    // sinon, envoyer directement la ligne
                    else
                        envoie_smtp (bufin);

                    // et lire la rponse du serveur
                    lire_smtp ();
                }
                // passage  la ligne suivante
                while (fgets (bufin, 80, fich_auth));
            }

            // librer le fichier d'authentification smtp
            fclose (fich_auth);
        }
        else
        {
            // message d'erreur
            // "Fichier %s non trouv"
            aff_err_arg ("FICH_ABSENT", carinfo);
            retour = 0;
        }
    }
    // sinon simple connexion  un serveur smtp sans authentification
    else
    {
        // parcourir le nom du serveur smtp
        carinfo = infoserveur;

        while (*carinfo > ' ')
            carinfo ++;

        // si le nom du serveur smtp est suivi d'autres informations
        if (*carinfo)
        {
            // terminer le nom du serveur smtp
            *carinfo = '\0';

            // se positionner sur le numro de port ventuel
            do
                carinfo++;
            while (*carinfo == ' ' || *carinfo == '\t');
        }

        // si un numro de port semble prsent
        if (*carinfo)
            // le rcuprer
            port = atoi (carinfo);

        // si numro de port trouv ou si on a conserv le port par dfaut
        if (port)
            // tablir la connexion smtp avec le port choisi
            retour = connect_serv_smtp (infoserveur, port);

        // sinon message d'erreur
        else
        {
            // "Numro de port smtp invalide"
            affiche_err ("PORT_SMTP_INVALIDE");
            retour = 0;
        }

        // si la connexion smtp s'est bien droule
        if (retour)
        {
            // gnrer le message  helo nom_de_calculateur
            strcpy (bufw, "HELO ");
            gethostname (bufw + 5, sizeof (bufw) - 5);

            // identification de l'ordinateur connect
            envoie_smtp (bufw);
            lire_smtp ();
        }
    }

    // renvoyer le code de retour de la fonction
    return (retour);
}



/* connexion au serveur smtp */

int connect_serv_smtp (char *serveur, int port_smtp)
{
    struct hostent *HostEnt;      // description du host serveur
    struct sockaddr_in serv_addr; // addresse du serveur


#ifdef DEBUG
    printf ("Appel de connect_serv_smtp (%s, %d)\n", serveur, port_smtp);
#endif

    // rcupration adresse du serveur  partir de son nom
    HostEnt = gethostbyname (serveur);

    if (HostEnt == NULL)
    {
        // "Serveur smtp non trouv"
        affiche_err ("SERV_SMTP_ABSENT");
        return (0);
    }

    // initialisation structure serv_adr
    memset (&serv_addr, 0, sizeof (serv_addr)); // init serv_addr

    memcpy (&serv_addr.sin_addr, HostEnt->h_addr, HostEnt->h_length);

    serv_addr.sin_port = htons (port_smtp);     // port smtp utilis
    serv_addr.sin_family = AF_INET;             // AF_*** : INET=internet

    // cration de la socket
    if ((smtp = socket (AF_INET, SOCK_STREAM, 0)) < 0)
    {
        // "Problme cration socket client smtp"
        affiche_err ("CREAT_SOCKET_SMTP");
        return (0);
    }

    // requte de connexion
    if (connect (smtp, (struct sockaddr*) &serv_addr, sizeof (serv_addr)) < 0)
    {
        // "Problme demande de connexion"
        // "peut tre due  une connexion Internet par proxy"
        affiche_err ("PB_ACCES_SERV-1");
        affiche_err ("PB_ACCES_SERV-2");
        return (0);
    }
    else
    {
        // attendre la rponse du serveur smtp
        lire_smtp (smtp);
        return (1);
    }
}



/* teste si la ligne passe en paramtre commence par un mot cl particulier */

int startligne (char *ligne, char *motcle)
{
    int i;

    i = 0;

    // on teste caractre par caractre en ignorant la casse
    while (motcle [i] && tolower (ligne [i]) == tolower (motcle [i]))
        i++;

    // le mot cl doit tre suivi d'un blanc dans la ligne lue
    return (ligne [i] == ' ');
}



/* Ecriture de donnes
   envoie la chaine passe en paramtre suivie de \r\n
*/

void envoie_smtp (char *buffer)
{
#ifdef DEBUG
    putchar ('>');
    puts (buffer);
    fflush (stdout);
#endif

    write (smtp, buffer, strlen (buffer));
    write (smtp, "\r\n", 2);
}



/* Lecture d'une ligne de donnes
   la lecture s'arrte sur un caractre de passage  la ligne
*/

void lire_smtp ()
{
    int posbuf;


    // initialisation
    posbuf = 0;

    // lecture
    do
        read (smtp, buf_lect + posbuf, 1);
    while (buf_lect [posbuf++] != '\n' && posbuf < sz_buflect);

#ifdef DEBUG
    buf_lect [posbuf - 1] = '\0';
    putchar ('<');
    puts (buf_lect);

    // si encodage base64, on affiche aussi la donne en clair
    if (startligne (buf_lect, "334"))
    {
        decode64 (buf_lect + 4);
        puts (buf_lect);
    }

    fflush (stdout);
#endif
}



/* dconnexion propre du serveur smtp */

void deconnect_smtp ()
{
    // envoi du message de deconnexion
    envoie_smtp ("QUIT");

    // lecture de l'acquittement
    lire_smtp ();

    // fermeture de la connexion
    shutdown (smtp, 2);
    close (smtp);
}
